--    Addon by Dutchmagé 
--    LUpdate: 23-03-2016
--    TODO:
--    Check if the party frame unbinds after choosen for Disable Auto Party Inv.

MagePortalWhispers = LibStub("AceAddon-3.0"):NewAddon("MagePortalWhispers", "AceConsole-3.0", "AceEvent-3.0", "AceTimer-3.0", "AceComm-3.0", "AceSerializer-3.0")

local L = LibStub("AceLocale-3.0"):GetLocale("MagePortalWhispers", true)
local LibC = LibStub:GetLibrary("LibCompress")
local LibCE = LibC:GetAddonEncodeTable()
local debug = false
local playername = ''
local default_whisper = "Sorry i don't give portals right now"
local player_class = UnitClass("player")

function isMage()
    return player_class:lower():match('mage')
end

-- get option value
local function GetGlobalOptionLocal(info)
	return MagePortalWhispers.db.global[info[#info]]
end

-- set option value
local function SetGlobalOptionLocal(info, value)
	if debug and MagePortalWhispers.db.global[info[#info]] ~= value then
		MagePortalWhispers:Printf("DEBUG: global option %s changed from '%s' to '%s'", info[#info], tostring(GuildRecr.db.global[info[#info]]), tostring(value))
	end
	MagePortalWhispers.db.global[info[#info]] = value
	LibStub("AceConfigRegistry-3.0"):NotifyChange("MagePortalWhispers")
end

-- declare defaults to be used in the DB
local defaults = {
	global = {
		enabled = isMage(),
		whisper = default_whisper,
        auto_party = false,
        display_whisper_overlay = false
	}
}

function ContainsPortalMessages(msg)
  return (msg:lower():match('portal') or 
    msg:lower():match('portals') or 
    msg:lower():match('teleport') or 
    msg:lower():match('teleports'))
end

-- a filter to hide all yelled messaged containing certain text
function NoWhisperWindow(self,event,msg)
  return ContainsPortalMessages(msg)
end

function InitTextFrame()
    MOD_TextFrame = CreateFrame("Frame");
    MOD_TextFrame:ClearAllPoints();
    MOD_TextFrame:SetHeight(300);
    MOD_TextFrame:SetWidth(300);
    MOD_TextFrame:SetScript("OnUpdate", MOD_TextFrame_OnUpdate);
    MOD_TextFrame:Hide();
    MOD_TextFrame.text = MOD_TextFrame:CreateFontString(nil, "BACKGROUND", "PVPInfoTextFont");
    MOD_TextFrame.text:SetAllPoints();
    MOD_TextFrame:SetPoint("CENTER", 0, 200);
    MOD_TextFrameTime = 0;
end

function MOD_TextFrame_OnUpdate()
  if (MOD_TextFrameTime < GetTime() - 3) then
    local alpha = MOD_TextFrame:GetAlpha();
    if (alpha ~= 0) then MOD_TextFrame:SetAlpha(alpha - .05); end
    if (alpha == 0) then MOD_TextFrame:Hide(); end
  end
end

function MOD_TextMessage(message)
      MOD_TextFrame.text:SetText(message);
      MOD_TextFrame:SetAlpha(1);
      MOD_TextFrame:Show();
      MOD_TextFrameTime = GetTime();
end

-- Option menu (Interface-->Settings)
local options = {
    name = "Mage Portal Whisper - by Dutchmagé",
	type = "group",
	childGroups = "tab",
	args = {
		general_tab = {
			name = "General",
			type = "group",
			order = 10,
			args = {
				enabled = {
					type = "toggle",
					order = 2,
					name = "Enable Addon",
					desc = "Enable or disable addon functionality.",
					width = "full",
					get =	GetGlobalOptionLocal,
					set =	function (info, value)
								SetGlobalOptionLocal(info, value)
							end
				},
                display_whisper_overlay = 
                {
                   type = "toggle",
					order = 3,
					name = "Enable whisper from player in overlay",
					desc = "Enable or disable whisper in overlay.",
					width = "full",
					get =	GetGlobalOptionLocal,
					set =	function (info, value)
								SetGlobalOptionLocal(info, value)
							end   
                },
                auto_party = 
                {
                   type = "toggle",
					order = 4,
					name = "Enable auto accept party invites.",
					desc = "Enable or disable auto accept party invites.",
					width = "full",
					get =	GetGlobalOptionLocal,
					set =	function (info, value)
								SetGlobalOptionLocal(info, value)
							end   
                },
				whisper = 
				{
					type = "input",
					order = 5,
					name = "Whisper",
					multiline = true,
					desc = "Custom whisper message",
					width = "full",
					get = GetGlobalOptionLocal,
					set = function (info, value)
							  SetGlobalOptionLocal(info, value)
						  end
				}
			}
		}
	}	
}

-- get server time in unix timestamp format
local function GetServerTime()
	local weekday, month, day, year = CalendarGetDate()
	local hours, minutes = GetGameTime()
	
	local timeset = { year = year, month = month, day = day, hour = hours, min = minutes }
	
	return time(timeset)
end

local function isempty(s)
  return s == nil or s == ''
end

function MagePortalWhispers:AcceptInvite()
   if MagePortalWhispers.db.global.auto_party then
        AcceptGroup()
        for i = 1, STATICPOPUP_NUMDIALOGS do
            local dialog = _G["StaticPopup" .. i]
            if dialog.which == "PARTY_INVITE" then
                dialog.inviteAccepted = 1
                break
            end
        end
        StaticPopup_Hide("PARTY_INVITE")
        f:UnregisterEvent("PARTY_MEMBERS_CHANGED")
    end
end

function MagePortalWhispers:AutoAccept()
    if MagePortalWhispers.db.global.auto_party then
        f = CreateFrame("Frame")
        f:RegisterEvent("PARTY_INVITE_REQUEST")
        f:SetScript("OnEvent", function(self, event, msg, sender)
              MagePortalWhispers:AcceptInvite()
          end)
    else
        f:UnregisterEvent("PARTY_INVITE_REQUEST")
    end
end

function MagePortalWhispers:WhisperBack()
	
	local dist, target
	dist, target = "WHISPER", dest
	
	playername = GetUnitName("player", true)
	f = CreateFrame("frame")
        
     f:RegisterEvent("CHAT_MSG_WHISPER")
    
        f:SetScript("OnEvent", function(self, event, msg, sender)
        
		if MagePortalWhispers.db.global.enabled then
            
            if not isempty(MagePortalWhispers.db.global.whisper) then
                wmessage = MagePortalWhispers.db.global.whisper
            end
			
            if ContainsPortalMessages(msg) then

                if not isempty(wmessage) then

                    if not debug then

                        -- if the player NOT matches yourself then..
                        if not sender:match(playername) then
                            if MagePortalWhispers.db.global.display_whisper_overlay then
                                InitTextFrame()
                                MOD_TextFrame:Hide();
                                MOD_TextMessage("|cffffff00"..sender.."|r: "..msg)
                            end
                            SendChatMessage(wmessage, "WHISPER", nil, sender);
                        else
                            MagePortalWhispers:Printf("|cff40c040Why are you whispering youself for portals? hic....|r")
                        end

                    else
                        -- if the player matches yourself then (DEBUGMODE)..
                        if sender:match(playername) then
                            if MagePortalWhispers.db.global.display_whisper_overlay then
                                -- Get the whisper of the player in an overlay and dissapear in 3 seconds.
                                MOD_TextMessage(msg)
                            end
                            -- Send a whisper to the player
                            SendChatMessage(string.gsub(wmessage, "portal", ""), "WHISPER", nil, sender);
                        end
                    end

                    else
                        MagePortalWhispers:Printf('|cff40c040Please define a Whisper text in /mpw or interface --> Mage Portal Whisper|r')
                    end

            end
		
		end
			
		do return end			
			
	end)
	
end

-- Called when the addon is disabled
function MagePortalWhispers:OnDisable()
	
	-- unregister events
	self:UnregisterAllEvents()
	
	-- unregister comm events
	self:UnregisterAllComm()
	
	self:UnregisterChatCommand("mpb")
	
end

-- Called when the addon is enabled
function MagePortalWhispers:OnEnable()
    
    if isMage() then
        MagePortalWhispers:WhisperBack()
        MagePortalWhispers:AutoAccept()
    else
        MagePortalWhispers:Printf("|cff0070ddthis addon can only be used on a Mage character|r")
    end
    
end

-- Process the slash command ("input" contains whatever follows the slash command)
function MagePortalWhispers:ConsoleCommand(input)
	-- show configuration window if no params given
	if not input or input:trim() == "" then
		InterfaceOptionsFrame_OpenToCategory(self.configFrame)
	end
	
	if input == "debug" then
		self:Print("DEBUG: enabled")
		debug = true
	end

	if input == "nodebug" then
		self:Print("DEBUG: disabled")
		debug = false
	end
    
end

-- Code that you want to run when the addon is first loaded goes here.
function MagePortalWhispers:OnInitialize()

    -- Only Init when the class is a mage.... !
    if isMage() then
        
        -- initialize saved variables
        self.db = LibStub("AceDB-3.0"):New("MagePortalWhispersDB", defaults, true)

        -- initialize configuration options
        LibStub("AceConfig-3.0"):RegisterOptionsTable("MagePortalWhispersDB", options)
        self.configFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("MagePortalWhispersDB", "Mage Portal Whisper");

        -- create LibDataBroker
        self.ldb = LibStub("LibDataBroker-1.1"):NewDataObject("Mage Portal Whisper", {
            type = "data source",
            text = "",
            label = "",
            icon = "Interface\\Icons\\Classicon_mage"
        })

        -- Only onetime init.
        InitTextFrame()
        
        --Block the crappy portals in channels.
        -- ChatFrame_AddMessageEventFilter("CHAT_MSG_CHANNEL", NoWhisperWindow)
        -- ChatFrame_AddMessageEventFilter("CHAT_MSG_SAY", NoWhisperWindow)
        -- ChatFrame_AddMessageEventFilter("CHAT_MSG_YELL", NoWhisperWindow)
        -- ChatFrame_AddMessageEventFilter("CHAT_MSG_WHISPER", NoWhisperWindow)
    
        
        self:RegisterChatCommand("mpw", "ConsoleCommand")
        
	end
    
end
